%PROFILE GENERATOR
%[X,V,A,J]=prfgen(parameters,dp,Fs)
%
% FOR STEP CASE
% parameters[Xf,t]
% FOR RAMP CASE
% parameters[Xf,Vm]
% FOR TRAPSPEED CASE
% parameters[Xf, Vm, Am, Dm]
% FOR TRAPACC CASE
% parameters[Xf, Vm, Am, Dm, Jm]


function [X,V,A,J]=prfgen(pm,dp,Fs)

switch upper(dp)
    case 'STEP'
        Xf=pm(1);
        
        K=fix(Fs*pm(2));
        
        X=[0,Xf*ones(1,K)];
        V=diff([0,X]).*Fs;
        A=diff([0,V]).*Fs;
        J=diff([0,A]).*Fs;
        
        
    case 'RAMP'
        Xf=pm(1);
        Vm=pm(2)/Fs;
        
        K=fix(Xf/Vm);
        
        V=[0,Vm*ones(1,K)];
        X=cumsum(V);        
        A=diff([0,V]).*Fs^2;
        J=diff([0,A]).*Fs;          
        V=V.*Fs;
        
        
    case 'TRAPSPEED'
        Xf=pm(1);
        Vm=pm(2)/Fs;
        Am=pm(3)/Fs^2;
        Dm=pm(4)/Fs^2;       
        
        K1=fix(Vm/Am);
        K3=fix(-Vm/Dm);
        
        if(K1<1)K1=1;
        end
        if(K3<1)K3=1;
        end
        
        if(Vm*(K1+K3)/2>Xf)
%             Vm=2*Xf/(K1+K3);
            K2=0;
        else
            K2=fix(Xf/Vm-(K1+K3)/2);
        end
        
        %REZIZEING PARAMETERS FOR ZERO ERROR GENERATED PROFILE
        Vm=2*Xf/(K1+2*K2+K3);
        Am=Vm/K1;
        Dm=-Vm/K3;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        J=zeros(1,K1+K2+K3+1);
        if(K2==0)
            J([2,K1+1])=[Am,-Am+Dm];
        else
            J([2,K1+1,K1+K2+1])=[Am,-Am,Dm];           
        end
        J(K1+K2+K3+1)=Am;
        A=cumsum(J);
        V=cumsum(A);
        X=cumsum(V);
        
        V=V.*Fs;
        A=A.*Fs^2;
        J=J.*Fs^3;
        
    case 'TRAPACC'
        Xf=pm(1);
        Vm=pm(2)/Fs;
        Am=pm(3)/Fs^2;
        Dm=pm(4)/Fs^2;
        Jm=pm(5)/Fs^3;
        
        K1=fix(Am/Jm);
                        
        if(K1<1)
            K1=1;
        end
        K3=K1;
        
        if(Am*K1>Vm)
            K2=0;
        else
            K2=fix(Vm/Am-K1);
        end
        
        K5=fix(Dm/-Jm);
        if(K5<1)
            K5=1;
        end
        K7=K5;
        
        if(-Dm*K5>Vm)
            K6=0;
        else
            K6=fix(-Vm/Dm-K5);
        end
        
        
        if(Vm*(2*K1+K2+2*K5+K6)/2>Xf)
            K4=0;
        else
            K4=fix(Xf/Vm-(2*K1+K2+2*K5+K6)/2);
        end
        
        Vm=2*Xf/(2*K1+K2+2*K5+K6+2*K4)*Fs;
        Am=Vm/(K1+K2)*Fs;
%         Dm=-Vm/(K5+K6);
        Jm=Am/K1*Fs;
        
        DJ=zeros(1,K1+K2+K3+K4+K5+K6+K7+1);
        
        DJ(2)=DJ(2)+Jm;
        DJ(K1+1)=DJ(K1+1)-Jm;
        DJ(K1+K2+1)=DJ(K1+K2+1)-Jm;
        DJ(K1+K2+K3+1)=DJ(K1+K2+K3+1)+Jm;
        DJ(K1+K2+K3+K4+1)=DJ(K1+K2+K3+K4+1)-Jm;
        DJ(K1+K2+K3+K4+K5+1)=DJ(K1+K2+K3+K4+K5+1)+Jm;
        DJ(K1+K2+K3+K4+K5+K6+1)=DJ(K1+K2+K3+K4+K5+K6+1)+Jm;
        
%         if(K4==0)
%             DJ([2,K1+1,K1+K2+1,K1+K2+K3+1,K1+K2+K3+K4+K5+1,K1+K2+K3+K4+K5+K6+1])...
%                 =[Jm,-Jm,-Jm,0,Jm,Jm];            
%         else
%             DJ([2,K1+1,K1+K2+1,K1+K2+K3+1,K1+K2+K3+K4+1,K1+K2+K3+K4+K5+1,K1+K2+K3+K4+K5+K6+1])...
%                 =[Jm,-Jm,-Jm,Jm,-Jm,Jm,Jm];   
%         end
%         
        J=cumsum(DJ);
        A=cumsum(J);
        V=cumsum(A)./Fs^2;       
        %%%minimizing cumulative errors
        V(K1+K2+K3+1:K1+K2+K3+K4)=V(K1+K2+K3);
        
        V(K1+K2+K3+K4+1:K1+K2+K3+K4+K5+K6+K7+1)=V(K1+K2+K3+1:-1:1);
        
%         V=V./max(abs(V)).*Vm;
        
        X=cumsum(V)./Fs;
         X=X./max(abs(X)).*abs(Xf);
        
        V=V;
        A=A./Fs;
        J=J;       
    case 'TRAPACCSJ'
        Xf=pm(1);
        Vm=pm(2)/Fs;
        Am=pm(3)/Fs^2;
        Dm=pm(4)/Fs^2;
        Jm=pm(5)/Fs^3;
        
        K1=fix(3*Am/2/Jm);
                        
        if(K1<1)
            K1=1;
        end
        K3=K1;
        
        if(Am*K1>Vm)
            K2=0;
        else
            K2=fix(Vm/Am-K1);
        end
        
        K5=fix(3*Dm/-Jm/2);
        if(K5<1)
            K5=1;
        end
        K7=K5;
        
        if(-Dm*K5>Vm)
            K6=0;
        else
            K6=fix(-Vm/Dm-K5);
        end
        
        
        if(Vm*(2*K1+K2+2*K5+K6)/2>Xf)
            K4=0;
        else
            K4=fix(Xf/Vm-(2*K1+K2+2*K5+K6)/2);
        end
        
        Vm=2*Xf/(2*K1+K2+2*K5+K6+2*K4)*Fs;
        Am=Vm/(K1+K2)*Fs;
%         Dm=-Vm/(K5+K6);
        Jm=3*Am/2/K1*Fs;
        
        J=zeros(1,K1+K2+K3+K4+K5+K6+K7+1);
        
        ki=0:K1-1;
        J(1:K1)=-4*Jm*ki.*(ki-K1)./K1^2;
        J(K1+K2+1:K1+K2+K3)=-J(1:K1);
        
        ki=0:K5-1;
        J(K1+K2+K3+K4+1:K1+K2+K3+K4+K5)=4*Jm*ki.*(ki-K5)./K5^2;        
        J(K1+K2+K3+K4+K5+K6+1:K1+K2+K3+K4+K5+K6+K7)=-J(K1+K2+K3+K4+1:K1+K2+K3+K4+K5);
        
        
        A=cumsum(J);
        V=cumsum(A);        
        X=cumsum(V)./Fs^3;
        
        V=V./Fs^2;
        A=A./Fs;
        J=J;     
    case 'CYCLOIDAL'
        Xf=pm(1);
        Vm=pm(2)/Fs;
        Am=pm(3)/Fs^2;
        Dm=pm(4)/Fs^2;
        Jm=pm(5)/Fs^3;
        
%         K1=fix(sqrt(2)*pi*Am/2/Jm);
        K1=fix(pi*Am/2/Jm);
                        
        if(K1<1)
            K1=1;
        end
        K3=K1;
        
        if(8*Jm*K1^2/pi^2>Vm)
            K2=0;
%             Jm=pi*Am/2/K1;            
            
            
        else
            K2=fix(Vm/Am-8*Jm*K1^2/Am/pi^2);
            
        end
        
        K5=K1;
        K6=K2;
        K7=K5;
        
%         
%         K5=fix(sqrt(2)*pi*Dm/-Jm/2);
%         if(K5<1)
%             K5=1;
%         end
%         K7=K5;
%         %%%%%%
%         if(8*Jm*K5^2/pi^2>Vm)
%             K6=0;
%             Jm=pi^2*Vm/(8*K5^2);
%         else
%             K6=fix(-Vm/Dm-8*Jm*K5^2/pi^2);
%         end
        
        
        
                
        if(Vm*(2*K1+K2+2*K5+K6)/2>Xf)
            K4=0;
            
        else
            K4=fix(Xf/Vm-(2*K1+K2+2*K5+K6)/2);
        end
            Vm=2*Xf/(2*K1+K2+2*K5+K6+2*K4)*Fs;
            Am=pi*Vm/(pi*K2+4*K1)*Fs;
            Jm=pi*Am/2/K1*Fs;
        
        
%         Vm=2*Xf/(2*K1+K2+2*K5+K6+2*K4)*Fs;
%         Jm=Jm*Fs^3;
        

        
        J=zeros(1,K1+K2+K3+K4+K5+K6+K7+1);
        
        ki=0:K1-1;
        J(1:K1)=Jm*cos(pi*ki/2/K1);
%         J(1:K1)=-4*Jm*ki.*(ki-K1)./K1^2;
        J(K1+K2+1:K1+K2+K3)=-Jm*sin(pi*ki/2/K1);
        
        ki=0:K5-1;
        J(K1+K2+K3+K4+1:K1+K2+K3+K4+K5)=-Jm*cos(pi*ki/2/K5);        
        J(K1+K2+K3+K4+K5+K6+1:K1+K2+K3+K4+K5+K6+K7)=-J(K1+K2+K3+K4+K5:-1:K1+K2+K3+K4+1);
        
        
        A=cumsum(J);
        V=cumsum(A);        
        
        %%%minimizing cumulative errors
        V(K1+K2+K3+1:K1+K2+K3+K4)=V(K1+K2+K3);        
        V(K1+K2+K3+K4+1:K1+K2+K3+K4+K5+K6+K7+1)=V(K1+K2+K3+1:-1:1);
        
        
        X=cumsum(V)./Fs^3;
        
        X=X./max(abs(X)).*abs(Xf);
        
        V=V./Fs^2;
        A=A./Fs;
        J=J;    
        
    otherwise
        
end
        